"""
团队管理-主播排行数据采集
"""

from DrissionPage import Chromium
from DrissionPage._pages.mix_tab import MixTab

from .._browser_utils import BrowserUtils
from .._utils import Utils
from ._dict import Dictionary
from ._utils import pick__custom_date_range


class Urls:
    anchor_rank = 'https://zhitou.feigua.cn/console/child/anchor'


class DataPacketUrls:
    anchor_rank = 'zhitou.feigua.cn/api/biz/v2/Anchor/Anchor/rank_data'


class Anchor:
    """主播排行, 如果已经打开了飞瓜智投页面将托管, 否则会打开新的标签页"""

    def __init__(self, browser: Chromium):
        self._browser = browser
        self._timeout = 15

    def _enter_check(self, page: MixTab, timeout: float = None):
        """页面进入检测, 如果失败将抛出异常"""

        _timeout = timeout if isinstance(timeout, (int, float)) else self._timeout

        page.listen.start(
            targets=DataPacketUrls.anchor_rank,
            method='GET',
            res_type='XHR',
        )
        if not Utils.same_url(Urls.anchor_rank, page.url):
            page.get(Urls.anchor_rank)
        else:
            page.refresh()

        packet = page.listen.wait(timeout=_timeout)
        if not packet:
            raise TimeoutError('页面打开后数据包获取超时, 可能访问异常')

    def get__anchor_rank__detail(self, begin_date: str, end_date: str):
        """获取主播排行-表格数据"""

        page = BrowserUtils.get__main_page(self._browser)
        self._enter_check(page)

        page.listen.start(
            targets=DataPacketUrls.anchor_rank, method='GET', res_type='XHR'
        )
        pick__custom_date_range(begin_date, end_date, page)
        packet = page.listen.wait(timeout=15)
        if not packet:
            raise TimeoutError('修改日期后数据包获取超时')

        resp: dict = packet.response.body
        data_list: list[dict] = resp.get('data')
        if isinstance(data_list, list) and len(data_list) == 0:
            raise ValueError('数据中的 data 字段为空列表, 可能没有数据')

        records = []
        for data in data_list:
            manager = data.get('manager').get('name')

            record = Utils.dict_mapping(data, Dictionary.anchor.anchor_rank__detail)
            record['主播'] = manager

            record = Utils.dict_format__float(
                record,
                fields=[
                    '销售额',
                    '小时成交额',
                    'UV价值',
                    '千川消耗(全域)',
                    '有效金额(含优惠券)',
                    '客单价',
                    'GPM',
                    '退款金额(店铺)',
                    '成交订单金额(含优惠券)',
                    '有效金额(店铺)',
                    '成交订单金额(店铺)',
                    '退款金额(含优惠券)',
                    '总投放消耗',
                    '优惠券金额(全域)',
                    '总成交优惠券金额',
                ],
            )
            record['直播时长'] = Utils.seconds_to_time(record['直播时长(秒)'])
            record['平均停留时长'] = Utils.seconds_to_time(record['平均停留时长(秒)'])

            records.append(record)

        return records

    def get__field_control_rank__detail(self, begin_date: str, end_date: str):
        """获取场控排行-表格数据"""

        page = BrowserUtils.get__main_page(self._browser)
        self._enter_check(page)

        # ========== 切换到场控标签页 ==========
        page.listen.start(
            targets=DataPacketUrls.anchor_rank, method='GET', res_type='XHR'
        )
        target_tab_ele = page.ele(
            'c:div[data-action_name="Team_Rank_FieldControl"]', timeout=3
        )
        if not target_tab_ele:
            raise RuntimeError('未找到 [场控] 标签页')

        page.listen.start(
            targets=DataPacketUrls.anchor_rank, method='GET', res_type='XHR'
        )
        target_tab_ele.click(by_js=True)
        if not page.listen.wait(timeout=15):
            raise RuntimeError('场控数据获取失败, 可能页面访问失败')
        # ========== 切换到场控标签页 ==========

        page.listen.start(
            targets=DataPacketUrls.anchor_rank, method='GET', res_type='XHR'
        )
        pick__custom_date_range(begin_date, end_date, page)
        packet = page.listen.wait(timeout=15)
        if not packet:
            raise TimeoutError('修改日期后数据包获取超时')

        resp: dict = packet.response.body
        data_list: list[dict] = resp.get('data')
        if isinstance(data_list, list) and len(data_list) == 0:
            raise ValueError('数据中的 data 字段为空列表, 可能没有数据')

        records = []
        for data in data_list:
            manager = data.get('manager').get('name')

            record = Utils.dict_mapping(data, Dictionary.anchor.anchor_rank__detail)
            record['场控'] = manager

            record = Utils.dict_format__float(
                record,
                fields=[
                    '销售额',
                    '小时成交额',
                    'UV价值',
                    '千川消耗(全域)',
                    '有效金额(含优惠券)',
                    '客单价',
                    'GPM',
                    '退款金额(店铺)',
                    '成交订单金额(含优惠券)',
                    '有效金额(店铺)',
                    '成交订单金额(店铺)',
                    '退款金额(含优惠券)',
                    '总投放消耗',
                    '优惠券金额(全域)',
                    '总成交优惠券金额',
                ],
            )
            record['直播时长'] = Utils.seconds_to_time(record['直播时长(秒)'])
            record['平均停留时长'] = Utils.seconds_to_time(record['平均停留时长(秒)'])

            records.append(record)

        return records



class Anchor:
    anchor_rank__detail = {
        '直播时长(秒)': 'live_duration',
        '观看人数': 'watch_uv',
        '销售额': 'gmv',
        '小时成交额': 'hour_gmv',
        'UV价值': 'uv_value',
        '千川消耗(全域)': 'uni_cost',
        '有效金额(含优惠券)': 'doudian_coupon_validate_gmv',
        '成交订单数': 'pay_order_num',
        '平均在线': 'avg_online_uv',
        '转粉率': 'conversion_fans_rate',
        '观看次数': 'watch_pv',
        '平均停留时长(秒)': 'avg_watch_duration',
        '互动率(频次)': 'watch_interact_cnt_ratio',
        '新增粉丝数': 'incr_fans_num',
        '评论数': 'comment_num',
        '成交人数': 'pay_ucnt',
        '成交转化率(人数)': 'pay_uv_rate',
        '客单价': 'atv',
        'GPM': 'gpm',
        '成交转化率(人次)': 'pay_pv_rate',
        '成交转化率(商品)': 'pay_product_rate',
        '商品成交人数': 'product_pay_uv',
        '退款金额(店铺)': 'doudian_refund_order_amount',
        '退款订单数(店铺)': 'doudian_refund_order_num',
        '成交订单数(店铺)': 'doudian_pay_order_num',
        '全场ROI(店铺)': 'doudian_pay_order_gmv_qianchuan_cost_ratio',
        '有效ROI(店铺)': 'doudian_valid_roi',
        '退款占比(店铺)': 'doudian_refund_gmv_ratio',
        '成交订单金额(含优惠券)': 'doudian_coupon_gmv',
        '有效金额(店铺)': 'doudian_valid_amount',
        '成交订单金额(店铺)': 'doudian_pay_order_gmv',
        '退单占比(店铺)': 'doudian_refund_order_ratio',
        '退款金额(含优惠券)': 'doudian_coupon_refund_gmv',
        '有效ROI(含优惠券)': 'doudian_coupon_validate_gmv_qianchuan_cost_ratio',
        '全场ROI(含优惠券)': 'doudian_coupon_gmv_qianchuan_cost_ratio',
        '总投放消耗': 'sum_cost',
        '全场ROI(券)': 'total_live_coupon_roi',
        '全场ROI': 'total_roi',
        '优惠券金额(全域)': 'qianchuan_uni_pay_coupon_gmv',
        '总成交优惠券金额': 'qianchuan_sum_pay_coupon_gmv',
        '非千川流量转化率': 'un_qianchuan_flow_pay_ratio',
        '千川ROI(全域)': 'pay_uni_roi',
    }


class Dictionary:
    anchor = Anchor()



"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-01-21
Author: Martian Bugs
Description: 数据采集器
"""

from DrissionPage import Chromium, ChromiumOptions

from .team_manage.team_manage import TeamManage


class Collector:
    """采集器. 使用之前请先调用 `connect_browser` 方法连接浏览器."""

    def __init__(self):
        self.browser = None

        self._team_manage = None

    def connect_browser(self, port: int):
        """
        连接浏览器

        Args:
            port: 浏览器调试端口号
        """

        chrome_options = ChromiumOptions(read_file=False)
        chrome_options.set_local_port(port=port)

        self.browser = Chromium(addr_or_opts=chrome_options)

    @property
    def team_manage(self):
        """团队管理"""

        if self._team_manage is None:
            self._team_manage = TeamManage(self.browser)

        return self._team_manage



from copy import deepcopy
from datetime import datetime, timedelta
from urllib.parse import urlparse


class Utils:
    @staticmethod
    def date_yesterday(pattern='%Y-%m-%d'):
        """
        获取前一天的日期

        Args:
            pattern: 日期格式
        """

        return Utils.date_calculate(days=1, pattern=pattern)

    @staticmethod
    def date_calculate(days: int, pattern='%Y-%m-%d'):
        """
        日期计算

        Args:
            days: 日期偏移量, 负数表示向后推
            pattern: 日期格式
        """

        return (datetime.now() - timedelta(days=days)).strftime(pattern)

    @staticmethod
    def date_diff_days(a: str, b: str, pattern='%Y-%m-%d'):
        """
        计算两个日期间隔的天数
        - 正数表示 a 日期在 b 日期之后

        Args:
            a: 日期字符串
            b: 日期字符串
        """

        a_dt = datetime.strptime(a, pattern)
        b_dt = datetime.strptime(b, pattern)

        return (a_dt - b_dt).days

    @staticmethod
    def seconds_to_time(seconds: int):
        """
        秒数转为时间字符串

        Args:
            seconds: 秒数
        """

        m, s = divmod(seconds, 60)
        h, m = divmod(m, 60)
        return f'{h:02d}:{m:02d}:{s:02d}'

    @staticmethod
    def same_url(a: str, b: str):
        """
        检查两个 url 是否域名及路径是否一致

        Args:
            a: 第一个 url
            b: 第二个 url
        Returns:
            是否一致
        """

        a_result = urlparse(a)
        b_result = urlparse(b)

        is_same = a_result.netloc == b_result.netloc and a_result.path == b_result.path

        return is_same

    @staticmethod
    def dict_mapping(data: dict, dict_table: dict[str, str]):
        """
        字典表字段映射

        Args:
            data: 待映射的字典
            dict_table: 字典表
        """

        result = {}
        for text, key in dict_table.items():
            result[text] = data.get(key)

        return result

    @staticmethod
    def dict_format__float(data: dict, fields: list[str] = None, precision: int = 2):
        """
        将字典数据中的指定字段格式化为 float 类型

        Args:
            data: 待格式化的字典
            fields: 需要格式化的字段列表, 如果为 None, 则表示所有字段
            precision: 保留小数位数, 默认 2 位
        Returns:
            格式化后的字典
        """

        _fields = fields if fields and isinstance(fields, list) else data.keys()

        _data = deepcopy(data)
        for field in _fields:
            if field not in _data:
                continue

            value = _data[field]
            if not isinstance(value, (int, float)):
                continue

            _data[field] = value / 10**precision

        return _data

    @staticmethod
    def dict_format__round(data: dict, fields: list[str] = None, precision: int = 2):
        """
        将字典数据中的指定字段作四舍五入处理

        Args:
            data: 待格式化的字典
            fields: 需要格式化的字段列表, 如果为 None, 则表示所有字段
            precision: 保留小数位数, 默认 2 位
        Returns:
            格式化后的字典
        """

        _fields = fields if fields and isinstance(fields, list) else data.keys()

        _data = deepcopy(data)
        for field in _fields:
            if field not in _data:
                continue

            value = _data[field]
            if not isinstance(value, float):
                continue

            _data[field] = round(value, precision)

        return _data

    @staticmethod
    def dict_format__ratio(data: dict, fields: list[str] = None, ratio: int = 2):
        """
        将字典数据中的指定字段转为比率, 例如百分比/千分比等

        Args:
            data: 待格式化的字典
            fields: 需要格式化的字段列表, 如果为 None, 则表示所有字段
            ratio: 比率, 默认 2 及百分比
        Returns:
            格式化后的字典
        """

        _fields = fields if fields and isinstance(fields, list) else data.keys()

        _data = deepcopy(data)
        for field in _fields:
            if field not in _data:
                continue

            value = _data[field]
            if not isinstance(value, (int, float)):
                continue

            _data[field] = value * (10**ratio)

        return _data

    @staticmethod
    def dict_format__strip(
        data: dict,
        fields: list[str] = None,
        prefix: list[str] = None,
        suffix: list[str] = None,
    ):
        """
        格式化字典数据中的指定字段, 去除前后空格及指定前后缀

        Args:
            data: 待格式化的字典
            fields: 需要格式化的字段列表, 如果为 None, 则表示所有字段
            prefix: 需要去除的前缀列表
            suffix: 需要去除的后缀列表
        Returns:
            格式化后的字典
        """

        _fields = fields if fields and isinstance(fields, list) else data.keys()

        _data = deepcopy(data)
        for field in _fields:
            if field not in _data:
                continue

            value = _data[field]
            if not isinstance(value, str):
                continue

            value = value.strip()
            if prefix and isinstance(prefix, list):
                for c in prefix:
                    value = value.lstrip(c)

            if suffix and isinstance(suffix, list):
                for c in suffix:
                    value = value.rstrip(c)

            _data[field] = value

        return _data

    @staticmethod
    def dict_format__number(data: dict, fields: list[str] = None):
        """
        格式化字典数据中的指定字段, 将字符串转为数字

        Args:
            data: 待格式化的字典
            fields: 需要格式化的字段列表, 如果为 None, 则表示所有字段
        Returns:
            格式化后的字典
        """

        _fields = fields if fields and isinstance(fields, list) else data.keys()

        _data = deepcopy(data)
        for field in _fields:
            if field not in _data:
                continue

            value = _data[field]
            if not isinstance(value, str):
                continue

            try:
                value = value.replace(',', '')
                value = float(value) if '.' in value else int(value)
            except ValueError:
                continue

            _data[field] = value

        return _data



"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-01-21
Author: Martian Bugs
Description: 团队管理数据采集
"""

from DrissionPage import Chromium

from .anchor import Anchor


class TeamManage:
    def __init__(self, browser: Chromium):
        self._browser = browser

        self._anchor = None

    @property
    def anchor(self):
        """主播排行"""

        if not self._anchor:
            self._anchor = Anchor(self._browser)

        return self._anchor



"""
浏览器工具集
"""

from DrissionPage import Chromium


class BrowserUtils:
    @staticmethod
    def get__main_page(browser: Chromium):
        """
        获取主要页面的浏览器标签页
        - 如果没有找到，则新建一个标签页并返回
        """

        page = browser.get_tab(title='飞瓜智投')
        if not page:
            page = browser.new_tab()
        else:
            page.set.activate()

        return page



from time import sleep

from DrissionPage._pages.mix_tab import MixTab
from DrissionPage.common import Keys


def pick__custom_date_range(begin_date: str, end_date: str, page: MixTab):
    """
    修改自定义日期范围

    Args:
        begin_date: 开始日期
        end_date: 结束日期
        page: 网页对象
    """

    mapping = [['开始日期', begin_date], ['结束日期', end_date]]
    for item in mapping:
        date_input_ele = page.ele(f'c:input[placeholder="{item[0]}"]', timeout=3)
        if not date_input_ele:
            raise RuntimeError(f'未找到 [{item[0]}] 输入框')

        date_input_ele.input(item[1] + Keys.ENTER, clear=True)
        sleep(0.5)
